PROGRAM testrealloc

  USE errorhandling
  USE reallocate_mod

  IMPLICIT NONE

  CHARACTER(LEN=5), DIMENSION(5)  :: char_ref = (/"one  ", "two  ", "three", "four ", "five "/)
  CHARACTER(LEN=5), POINTER, DIMENSION(:) :: char_array

  REAL(KIND=DOUBLE), DIMENSION(5) :: dble_ref = (/ 1.0, 2.0, 3.0, 4.0, 5.0 /)
  REAL(KIND=DOUBLE), POINTER, DIMENSION(:) :: dble_array

  REAL(KIND=single), DIMENSION(5) :: sing_ref = (/ 1.0, 2.0, 3.0, 4.0, 5.0 /)
  REAL(KIND=single), POINTER, DIMENSION(:) :: sing_array

  INTEGER(KIND=int32), DIMENSION(5) :: int32_ref = (/ 1, 2, 3, 4, 5 /)
  INTEGER(KIND=int32), POINTER, DIMENSION(:) :: int32_array

  INTEGER(KIND=int16), DIMENSION(5) :: int16_ref = (/ 1, 2, 3, 4, 5 /)
  INTEGER(KIND=int16), POINTER, DIMENSION(:) :: int16_array

  INTEGER(KIND=int8), DIMENSION(5) :: int8_ref = (/ 1, 2, 3, 4, 5 /)
  INTEGER(KIND=int8), POINTER, DIMENSION(:) :: int8_array


! Test for character arrays

  ALLOCATE(char_array(0:4))
  char_array(0:4) = char_ref(1:5)

  CALL reallocate(char_array, 10)
  IF( SIZE(char_array) /= 10 ) CALL error("CharSize", "Wrong size for character array")
  IF( ANY( char_array(0:4) /= char_ref(1:5) ) )THEN
     CALL error("CharInfo", "Data lost in the character array")
  END IF

  CALL reallocate(char_array, 4)
  IF( SIZE(char_array) /= 4 ) CALL error("CharSize", "Wrong size for character array")
  IF( ANY( char_array(0:3) /= char_ref(1:4) ) )THEN
     CALL error("CharInfo", "Data lost in the character array")
  END IF

! Test for real arrays

  ALLOCATE(dble_array(0:4))
  dble_array(0:4) = dble_ref(1:5)

  CALL reallocate(dble_array, 10)
  IF( SIZE(dble_array) /= 10 ) CALL error("DbleSize", "Wrong size for double array")
  IF( ANY( dble_array(0:4) /= dble_ref(1:5) ) )THEN
     CALL error("DbleInfo", "Data lost in the double array")
  END IF

  CALL reallocate(dble_array, 4)
  IF( SIZE(dble_array) /= 4 ) CALL error("DbleSize", "Wrong size for double array")
  IF( ANY( dble_array(0:3) /= dble_ref(1:4) ) )THEN
     CALL error("DbleInfo", "Data lost in the double array")
  END IF


  ALLOCATE(sing_array(0:4))
  sing_array(0:4) = sing_ref(1:5)

  CALL reallocate(sing_array, 10)
  IF( SIZE(sing_array) /= 10 ) CALL error("SingSize", "Wrong size for single array")
  IF( ANY( sing_array(0:4) /= sing_ref(1:5) ) )THEN
     CALL error("SingInfo", "Data lost in the single array")
  END IF

  CALL reallocate(sing_array, 4)
  IF( SIZE(sing_array) /= 4 ) CALL error("SingSize", "Wrong size for single array")
  IF( ANY( sing_array(0:3) /= sing_ref(1:4) ) )THEN
     CALL error("SingInfo", "Data lost in the single array")
  END IF

! Test for integer arrays

  ALLOCATE(int32_array(0:4))
  int32_array(0:4) = int32_ref(1:5)

  CALL reallocate(int32_array, 10)
  IF( SIZE(int32_array) /= 10 ) CALL error("Int32Size", "Wrong size for integer 32 array")
  IF( ANY( int32_array(0:4) /= int32_ref(1:5) ) )THEN
     CALL error("Int32Info", "Data lost in the integer 32 array")
  END IF

  CALL reallocate(int32_array, 4)
  IF( SIZE(int32_array) /= 4 ) CALL error("Int32Size", "Wrong size for integer 32 array")
  IF( ANY( int32_array(0:3) /= int32_ref(1:4) ) )THEN
     CALL error("Int32Info", "Data lost in the integer 32 array")
  END IF


  ALLOCATE(int16_array(0:4))
  int16_array(0:4) = int16_ref(1:5)

  CALL reallocate(int16_array, 10)
  IF( SIZE(int16_array) /= 10 ) CALL error("Int16Size", "Wrong size for integer 16 array")
  IF( ANY( int16_array(0:4) /= int16_ref(1:5) ) )THEN
     CALL error("Int16Info", "Data lost in the integer 16 array")
  END IF

  CALL reallocate(int16_array, 4)
  IF( SIZE(int16_array) /= 4 ) CALL error("Int16Size", "Wrong size for integer 16 array")
  IF( ANY( int16_array(0:3) /= int16_ref(1:4) ) )THEN
     CALL error("Int16Info", "Data lost in the integer 16 array")
  END IF


  ALLOCATE(int8_array(0:4))
  int8_array(0:4) = int8_ref(1:5)

  CALL reallocate(int8_array, 10)
  IF( SIZE(int8_array) /= 10 ) CALL error("Int8Size", "Wrong size for integer 8 array")
  IF( ANY( int8_array(0:4) /= int8_ref(1:5) ) )THEN
     CALL error("Int8Info", "Data lost in the integer 8 array")
  END IF

  CALL reallocate(int8_array, 4)
  IF( SIZE(int8_array) /= 4 ) CALL error("Int8Size", "Wrong size for integer 8 array")
  IF( ANY( int8_array(0:3) /= int8_ref(1:4) ) )THEN
     CALL error("Int8Info", "Data lost in the integer 8 array")
  END IF

END PROGRAM testrealloc
